@@ -1,19 +1,19 @@
 `networklevel` <-
 function(web, index="ALL", ISAmethod="Bluethgen", SAmethod="Bluethgen", extinctmethod="r",
     nrep=100, plot.it.extinction=FALSE, plot.it.dd=FALSE, CCfun=median, dist="horn",
-    normalise=TRUE){
+    normalise=TRUE, nest.weighted=FALSE, empty.web=TRUE){
     ##
     ## web         interaction matrix, with lower trophic level in rows, higher in columns
     ##
-    web <- empty(web)
-    if (nrow(web) < 2 | ncol(web) <2) stop("Web is too small to calculate any reasonable index.")
+    if(empty.web) {web <- empty(web)}
+    web.e <- empty(web) # emptied web for some indices 
+    if (nrow(web) < 2 | ncol(web) <2) warning("Web is really too small to calculate any reasonable index. You will get the values nonetheless, but I wouldn't put any faith in them!")
 
     if (any(index %in% "ALL")) index <- c("number of species", "links per species",
           "connectance", "linkage density", "web asymmetry",
           "number of compartments", "generality", "vulnerability", "interaction evenness",
           "compartment diversity", "cluster coefficient", "H2", "ISA", "SA",
-          "extinction slope", "degreedistribution", "niche overlap", "mean number of shared hosts", 
-          "C-score", "togetherness", "V-ratio", "nestedness")
+          "extinction slope", "degreedistribution", "niche overlap", "mean number of shared hosts",  "C-score", "togetherness", "V-ratio", "nestedness", "nestedness.corso", "discrepancy")
     out <- list()
 
     # set up enough panals for plotting:
@@ -66,11 +66,12 @@ function(web, index="ALL", ISAmethod="Bluethgen", SAmethod="Bluethgen", extinctm
         # linkage density
         LD_q <- 0.5*(V+G)
         if ("linkage density" %in% index) out$"linkage density"=LD_q
-        #LD_qs <- LD_q/(NROW(web)+NCOL(web)) # according to Jason's text
+        #LD_qs <- LD_q/(NROW(web)+NCOL(web)) # "weighted food web connectance", according to Jason's appendix
         # interaction evenness
         p_i.mat <- web/sum(web)
-        IE <- -sum(p_i.mat*log2(p_i.mat), na.rm=TRUE)/log2(sum(web!=0))
-
+        SH <- -sum(p_i.mat*log(p_i.mat), na.rm=TRUE)
+        IE <- SH/log(prod(dim(web))) #log(sum(web>0))
+        
         evenness <- function(web){
             # calculates evenness of the numbers of individuals of different species in
             # a community, NOT according to formula in Mller et al. 1999, J. Anim. Ecol,
@@ -82,6 +83,7 @@ function(web, index="ALL", ISAmethod="Bluethgen", SAmethod="Bluethgen", extinctm
 
         E <- evenness(web)
         if ("interaction evenness" %in% index){
+            out$"Shannon diversity" <- SH
             out$"interaction evenness"=IE
             out$"Alatalo interaction evenness"=E
         }
@@ -106,7 +108,7 @@ function(web, index="ALL", ISAmethod="Bluethgen", SAmethod="Bluethgen", extinctm
           CD
         }
        
-        comps <- compart(web)
+        comps <- compart(web.e)
         if (class(comps)=="try-error") {
             ncompart <- compdiv <- NA
         } else  {
@@ -156,8 +158,10 @@ function(web, index="ALL", ISAmethod="Bluethgen", SAmethod="Bluethgen", extinctm
 
     #---------------------------------------------------------------------------
     # Blthgen's specialisation/non-conformity index
-    if ("H2" %in% index) H2 <- as.numeric(H2fun(web)[1]) #1.element is the standardised H2 prime
+    if ("H2" %in% index){
+        H2 <- as.numeric(H2fun(web)[1]) #1.element is the standardised H2 prime
         out$"H2"= ifelse(H2<0, 0, H2)
+    }
     #---------------------------------------------------------------------------
     # web asymmetry (Blthgen et al. 2007, Fig. S2)
     if (any(c("SA", "ISA", "web asymmetry") %in% index)){
@@ -166,10 +170,15 @@ function(web, index="ALL", ISAmethod="Bluethgen", SAmethod="Bluethgen", extinctm
 
         #----------------------------------------------------------------------------
         # Dependence asymmetry (Bascompte et al. 2006; Blthgen et al. 2007, Fig. S2)
-        depL <- web/matrix(rowSums(web), nrow=NROW(web), ncol=NCOL(web), byrow=FALSE)
-        depH <- web/matrix(colSums(web), nrow=NROW(web), ncol=NCOL(web), byrow=TRUE)
+        depL <- web.e/matrix(rowSums(web.e), nrow=NROW(web.e), ncol=NCOL(web.e), byrow=FALSE)
+        depH <- web.e/matrix(colSums(web.e), nrow=NROW(web.e), ncol=NCOL(web.e), byrow=TRUE)
 
-        if (ISAmethod=="Bascompte" & "ISA" %in% index)  out$"dependence asymmetry"=mean(abs(depL-depH)/max(depL, depH))
+        if (ISAmethod=="Bascompte" & "ISA" %in% index) {
+            depMax <- depL
+            greaterindex <- depL < depH
+            depMax[greaterindex] <- depH[greaterindex]
+            out$"dependence asymmetry"=mean(abs(depL-depH)/depMax, na.rm=TRUE)
+        }
         if (ISAmethod=="Bluethgen" & "ISA" %in% index) {
             web2 <- web
             # delete cells for species encountered only once:
@@ -197,18 +206,20 @@ function(web, index="ALL", ISAmethod="Bluethgen", SAmethod="Bluethgen", extinctm
         # either as Blthgen et al: average weighted by number of interactions in the cell
         # or as mean of logarithms (since the dependencies follow a lognormal distribution)
 
-        di <- dfun(web)$dprime
-        dj <- dfun(t(web))$dprime
-        if (SAmethod=="log"){
-            lgmeani <- mean(log(di)); lgmeanj <- mean(log(dj))
-            SA <- (lgmeanj-lgmeani)/sum(lgmeani, lgmeanj)  # ij-sequence changed because log changes sequence, too
+        if ("SA" %in% index){
+          di <- dfun(web)$dprime  # plants
+          dj <- dfun(t(web))$dprime # pollinators
+          if (SAmethod=="log"){
+              lgmeani <- mean(log(di[di>0])); lgmeanj <- mean(log(dj[dj>0]))
+              SA <- (lgmeanj-lgmeani)/sum(lgmeani, lgmeanj)  # ij-sequence changed because log changes sequence, too
+          }
+          if (SAmethod=="Bluethgen"){
+              wmeani <- sum(di*rowSums(web.e))/sum(web.e)
+              wmeanj <- sum(dj*colSums(web.e))/sum(web.e)
+              SA <- (wmeanj-wmeani)/sum(wmeani, wmeanj) # positive values indicate more specialisation in the higher trophic level
+          }
+          out$"specialisation asymmetry"=SA
         }
-        if (SAmethod=="Bluethgen"){
-            wmeani <- sum(di*rowSums(web))/sum(web)
-            wmeanj <- sum(dj*colSums(web))/sum(web)
-            SA <- (wmeanj-wmeani)/sum(wmeani, wmeanj)
-        }
-        out$"specialisation asymmetry"=SA
 
     }
     #----------------------------------------------------------------------------
@@ -226,8 +237,11 @@ function(web, index="ALL", ISAmethod="Bluethgen", SAmethod="Bluethgen", extinctm
     #----------------------------------------------------------------------------
     # degree distribution fits:
     if ("degreedistribution" %in% index){
-        dd <- try(degreedistr(web, plot.it=plot.it.dd, pure.call=FALSE))
-
+        dd <- try(degreedistr(web.e, plot.it=plot.it.dd, pure.call=FALSE))
+        if (class(dd)=="try-error"){
+          dd$"lower trophic level dd fits" <- NA
+          dd$"higher trophic level dd fits" <- NA
+        }
         out$"degree distribution lower trophic level"=dd$"lower trophic level dd fits"
         out$"degree distribution higher trophic level"=dd$"higher trophic level dd fits"
 
@@ -239,8 +253,8 @@ function(web, index="ALL", ISAmethod="Bluethgen", SAmethod="Bluethgen", extinctm
     # to calculate dissimilarity between higher level species; similarity is simply
     # 1-dissimilarity:
     if ("niche overlap" %in% index) {
-      NOhigher <- mean(1-vegdist(t(web), method=dist))
-      NOlower <- mean(1-vegdist(web, method=dist))
+      NOhigher <- mean(1-vegdist(t(web.e), method=dist))
+      NOlower <- mean(1-vegdist(web.e, method=dist))
       out$"higher trophic level niche overlap" <- NOhigher
       out$"lower trophic level niche overlap" <- NOlower
     }
@@ -270,7 +284,20 @@ function(web, index="ALL", ISAmethod="Bluethgen", SAmethod="Bluethgen", extinctm
 
     #-------------------
     if ("nestedness" %in% index){
-      out$nestedness <- nestedness(web, null.models=FALSE)$temperature
+      nest <- try(nestedtemp(web)$statistic)
+      out$nestedness <- ifelse(class(nest)=="try-error", NA, nest)
+      # a fast implementation of nestedness by Jari Oksanen
+      #nestedness(web, null.models=FALSE)$temperature
+    }
+
+    #-------------------
+    if ("nestedness.corso" %in% index){
+      out$nestedness <- nestedness.corso(web.e, weighted=nest.weighted)
+    }
+
+    #-------------------
+    if ("discrepancy" %in% index){
+      out$discrepancy <- unname(discrepancy(web))
     }
 
     return(out)
